@@ -69,6 +69,7 @@ class Scraper(models.Model):
     WORK_STATUS_CHOICES = (
         ('R2', 'REVISION NEEDED (MAJOR)'),
         ('R1', 'REVISION NEEDED (MINOR)'),
+        ('UR', 'UNRESOLVED'),
         ('BR', 'BROKEN'),
         ('W', 'WORKING'),
         ('RC', 'RELEASE CANDIDATE'),
@@ -94,8 +95,9 @@ class Scraper(models.Model):
     )
     PAGINATION_TYPE = (
         ('N', 'NONE'),
-        ('R', 'RANGE_FUNCT'),
-        ('F', 'FREE_LIST'),
+        ('R', 'RANGE_FUNCT (+FOLLOW)'),
+        ('F', 'FREE_LIST (+FOLLOW)'),
+        ('O', 'FOLLOW'),
     )
     name = models.CharField(max_length=200)
     scraped_obj_class = models.ForeignKey(ScrapedObjClass)
@@ -112,6 +114,12 @@ class Scraper(models.Model):
     pagination_append_str = models.CharField(max_length=200, blank=True, help_text="Syntax: /somepartofurl/{page}/moreurlstuff.html")
     pagination_page_replace = models.TextField(blank=True, 
         help_text="RANGE_FUNCT: uses Python range funct., syntax: [start], stop[, step], FREE_LIST: 'Replace text 1', 'Some other text 2', 'Maybe a number 3', ...")
+    help_text = "Optional, follow links from a single non-paginated or all statically paginated (RANGE_FUNCT, FREE_LIST) main pages"
+    follow_pages_url_xpath = models.TextField(blank=True, help_text=help_text)
+    help_text = "Optional additional XPath for the page number, can be used in {follow_page} placeholder."
+    follow_pages_page_xpath = models.TextField(blank=True, help_text=help_text)
+    help_text = "Optionally limit number of pages to follow (default: follow until XPath fails)"
+    num_pages_follow = models.IntegerField(blank=True, null=True, help_text=help_text)
     last_scraper_save_alert_period = models.CharField(max_length=5, blank=True, 
         help_text="Optional, used for scraper monitoring with 'check_last_scraper_saves' management cmd, \
         syntax: [HOURS]h or [DAYS]d or [WEEKS]w (e.g. '6h', '5d', '2w')")
@@ -154,6 +162,9 @@ class Scraper(models.Model):
     
     def get_main_page_rpt(self):
         return self.requestpagetype_set.get(page_type='MP')
+    
+    def get_follow_page_rpts(self):
+        return self.requestpagetype_set.filter(page_type='FP')
 
     def get_detail_page_rpts(self):
         return s.requestpagetype_set.filter(~Q(page_type='MP'))
@@ -226,7 +237,7 @@ class Scraper(models.Model):
 
 @python_2_unicode_compatible
 class RequestPageType(models.Model):
-    TYPE_CHOICES = tuple([("MP", "Main Page")] + [("DP{n}".format(n=str(n)), "Detail Page {n}".format(n=str(n))) for n in list(range(1, 26))])
+    TYPE_CHOICES = tuple([("MP", "Main Page"), ("FP", "Follow Page"),] + [("DP{n}".format(n=str(n)), "Detail Page {n}".format(n=str(n))) for n in list(range(1, 26))])
     CONTENT_TYPE_CHOICES = (
         ('H', 'HTML'),
         ('X', 'XML'),
@@ -240,18 +251,19 @@ class RequestPageType(models.Model):
         ('GET', 'GET'),
         ('POST', 'POST'),
     )
-    page_type = models.CharField(max_length=3, choices=TYPE_CHOICES)
+    help_text = "One main page RPT, an optional follow page RPT (if follow pagination is used) and detail page RPTs for all DETAIL_PAGE_URLs"
+    page_type = models.CharField(max_length=3, choices=TYPE_CHOICES, help_text=help_text)
     scraped_obj_attr = models.ForeignKey(ScrapedObjAttr, blank=True, null=True, help_text="Empty for main page, attribute of type DETAIL_PAGE_URL scraped from main page for detail pages.")
     scraper = models.ForeignKey(Scraper)
     content_type = models.CharField(max_length=1, choices=CONTENT_TYPE_CHOICES, default='H', help_text="Data type format for scraped pages of page type (for JSON use JSONPath instead of XPath)")
     render_javascript = models.BooleanField(default=False, help_text="Render Javascript on pages (ScrapyJS/Splash deployment needed, careful: resource intense)")
     request_type = models.CharField(max_length=1, choices=REQUEST_TYPE_CHOICES, default='R', help_text="Normal (typically GET) request (default) or form request (typically POST), using Scrapys corresponding request classes (not used for checker).")
     method = models.CharField(max_length=10, choices=METHOD_CHOICES, default='GET', help_text="HTTP request via GET or POST.")
-    headers = models.TextField(blank=True, help_text='Optional HTTP headers sent with each request, provided as a JSON dict (e.g. {"Referer":"http://referer_url"}, use double quotes!)), can use {main page attribute} and {page} placeholders.')
-    body = models.TextField(blank=True, help_text="Optional HTTP message body provided as a unicode string, can use {main page attribute} and {page} placeholders.")
-    cookies = models.TextField(blank=True, help_text="Optional cookies as JSON dict (use double quotes!), can use {main page attribute} and {page} placeholders.")
+    headers = models.TextField(blank=True, help_text='Optional HTTP headers sent with each request, provided as a JSON dict (e.g. {"Referer":"http://referer_url"}, use double quotes!)), can use {main page attribute}, {page} and {follow_page} placeholders.')
+    body = models.TextField(blank=True, help_text="Optional HTTP message body provided as a unicode string, can use {main page attribute}, {page} and {follow_page} placeholders.")
+    cookies = models.TextField(blank=True, help_text="Optional cookies as JSON dict (use double quotes!), can use {main page attribute}, {page} and {follow_page} placeholders.")
     meta = models.TextField(blank=True, help_text="Optional Scrapy meta attributes as JSON dict (use double quotes!), see Scrapy docs for reference.")
-    form_data = models.TextField(blank=True, help_text="Optional HTML form data as JSON dict (use double quotes!), only used with FormRequest request type, can use {main page attribute} and {page} placeholders.")
+    form_data = models.TextField(blank=True, help_text="Optional HTML form data as JSON dict (use double quotes!), only used with FormRequest request type, can use {main page attribute}, {page} and {follow_page} placeholders.")
     dont_filter = models.BooleanField(default=False, help_text="Do not filter duplicate requests, useful for some scenarios with requests falsely marked as being duplicate (e.g. uniform URL + pagination by HTTP header).")
     comments = models.TextField(blank=True)
 
