@@ -27,9 +27,7 @@ from dynamic_scraper.utils import processors
 
 
 class DjangoSpider(DjangoBaseSpider):
-
-    mp_form_data = None
-    dp_form_data = {}
+    
     tmp_non_db_results = {}
     non_db_results = {}
     
@@ -89,16 +87,17 @@ class DjangoSpider(DjangoBaseSpider):
             '',
             'Options',
             '-------',
-            '-a do_action=(yes|no)                   Save output to DB, default: no (Test Mode)',
-            '-L LOG_LEVEL (scrapy option)            Setting the log level for both Scrapy and DDS',
-            '-a run_type=(TASK|SHELL)                Simulate task based scraper run, default: SHELL',
-            '-a max_items_read=[Int]                 Limit number of items to read',
-            '-a max_items_save=[Int]                 Limit number of items to save',
-            '-a max_pages_read=[Int]                 Limit number of pages to read',
-            '-a start_page=[PAGE]                    Start at page PAGE, e.g. 5, F',
-            '-a end_page=[PAGE]                      End scraping at page PAGE, e.g. 10, M',
-            '-a output_num_mp_response_bodies=[Int]  Output response body content of MP for debugging',
-            '-a output_num_dp_response_bodies=[Int]  Output response body content of DP for debugging',
+            '-a do_action=(yes|no)                       Save output to DB, default: no (Test Mode or File Output)',
+            '-L LOG_LEVEL (scrapy option)                Setting the log level for both Scrapy and DDS',
+            '-a run_type|rt=(TASK|SHELL)                 Simulate task based scraper run, default: SHELL',
+            '-a max_items_read|mir=[Int]                 Limit number of items to read',
+            '-a max_items_save|mis=[Int]                 Limit number of items to save',
+            '-a max_pages_read|mpr=[Int]                 Limit number of pages to read (static pagination)',
+            '-a start_page|sp=[PAGE]                     Start at page PAGE, e.g. 5, F (static pagination)',
+            '-a end_page|ep=[PAGE]                       End scraping at page PAGE, e.g. 10, M (static pagination)',
+            '-a num_pages_follow|npf=[Int]               Number of pages to follow (dynamic pagination)',
+            '-a output_num_mp_response_bodies|omp=[Int]  Output response body content of MP for debugging',
+            '-a output_num_dp_response_bodies|odb=[Int]  Output response body content of DP for debugging',
             '',
         )
         for out_str in out:
@@ -119,15 +118,13 @@ class DjangoSpider(DjangoBaseSpider):
                     msg = "Incorrect form_data attribute ({pt}): not a valid JSON dict!".format(pt=rpt.page_type)
                     self.dds_logger.error(msg)
                     raise CloseSpider()
-                if rpt.page_type == 'MP':
-                    self.mp_form_data = form_data
-                else:
-                    self.dp_form_data[rpt.page_type] = form_data
 
 
     def _set_config(self, **kwargs):
         log_msg = ""
-        #max_items_read 
+        #max_items_read|mir
+        if 'mir' in kwargs:
+            kwargs['max_items_read'] = kwargs['mir']
         if 'max_items_read' in kwargs:
             try:
                 self.conf['MAX_ITEMS_READ'] = int(kwargs['max_items_read'])
@@ -140,7 +137,9 @@ class DjangoSpider(DjangoBaseSpider):
             log_msg += "max_items_read " + str(self.conf['MAX_ITEMS_READ'])
         else:
             self.conf['MAX_ITEMS_READ'] = self.scraper.max_items_read
-        #max_items_save 
+        #max_items_save|mis
+        if 'mis' in kwargs:
+            kwargs['max_items_save'] = kwargs['mis']
         if 'max_items_save' in kwargs:
             try:
                 self.conf['MAX_ITEMS_SAVE'] = int(kwargs['max_items_save'])
@@ -153,7 +152,9 @@ class DjangoSpider(DjangoBaseSpider):
             log_msg += "max_items_save " + str(self.conf['MAX_ITEMS_SAVE'])
         else:
             self.conf['MAX_ITEMS_SAVE'] = self.scraper.max_items_save
-        #max_pages_read 
+        #max_pages_read|mpr
+        if 'mpr' in kwargs:
+            kwargs['max_pages_read'] = kwargs['mpr'] 
         if 'max_pages_read' in kwargs:
             try:
                 self.conf['MAX_PAGES_READ'] = int(kwargs['max_pages_read'])
@@ -166,17 +167,38 @@ class DjangoSpider(DjangoBaseSpider):
             log_msg += "max_pages_read " + str(self.conf['MAX_PAGES_READ'])
         else:
             self.conf['MAX_PAGES_READ'] = None
-        #start_page
+        #start_page|sp
+        if 'sp' in kwargs:
+            kwargs['start_page'] = kwargs['sp']
         if 'start_page' in kwargs:
             self.conf['START_PAGE'] = kwargs['start_page']
         else:
             self.conf['START_PAGE'] = None
-        #end_page
+        #end_page|ep
+        if 'ep' in kwargs:
+            kwargs['end_page'] = kwargs['ep']
         if 'end_page' in kwargs:
             self.conf['END_PAGE'] = kwargs['end_page']
         else:
             self.conf['END_PAGE'] = None
-        #output_num_mp_response_bodies
+        #num_pages_follow|npf
+        if 'npf' in kwargs:
+            kwargs['num_pages_follow'] = kwargs['npf']
+        if 'num_pages_follow' in kwargs:
+            try:
+                self.conf['NUM_PAGES_FOLLOW'] = int(kwargs['num_pages_follow'])
+            except ValueError:
+                msg = "You have to provide an integer value as num_pages_follow parameter!"
+                self.dds_logger.error(msg)
+                raise CloseSpider()
+            if len(log_msg) > 0:
+                log_msg += ", "
+            log_msg += "num_pages_follow " + str(self.conf['NUM_PAGES_FOLLOW'])
+        else:
+            self.conf['NUM_PAGES_FOLLOW'] = self.scraper.num_pages_follow
+        #output_num_mp_response_bodies|omp
+        if 'omp' in kwargs:
+            kwargs['output_num_mp_response_bodies'] = kwargs['omp']
         if 'output_num_mp_response_bodies' in kwargs:
             try:
                 self.conf['OUTPUT_NUM_MP_RESPONSE_BODIES'] = int(kwargs['output_num_mp_response_bodies'])
@@ -189,7 +211,9 @@ class DjangoSpider(DjangoBaseSpider):
             log_msg += "output_num_mp_response_bodies " + str(self.conf['OUTPUT_NUM_MP_RESPONSE_BODIES'])
         else:
             self.conf['OUTPUT_NUM_MP_RESPONSE_BODIES'] = 0
-        #output_num_dp_response_bodies
+        #output_num_dp_response_bodies|odp
+        if 'odp' in kwargs:
+            kwargs['output_num_dp_response_bodies'] = kwargs['odp']
         if 'output_num_dp_response_bodies' in kwargs:
             try:
                 self.conf['OUTPUT_NUM_DP_RESPONSE_BODIES'] = int(kwargs['output_num_dp_response_bodies'])
@@ -241,7 +265,7 @@ class DjangoSpider(DjangoBaseSpider):
     def _set_start_urls(self, scrape_url):
         self.start_urls = []
         
-        if self.scraper.pagination_type != 'N':
+        if self.scraper.pagination_type in ['R', 'F',]:
             if not self.scraper.pagination_page_replace:
                 msg = 'Please provide a pagination_page_replace context corresponding to pagination_type!'
                 self.dds_logger.error(msg)
@@ -274,7 +298,7 @@ class DjangoSpider(DjangoBaseSpider):
                 raise CloseSpider()
             pages = self.limit_page_nums(pages)
         
-        if self.scraper.pagination_type != 'N':
+        if self.scraper.pagination_type in ['R', 'F',]:
             append_str = self.scraper.pagination_append_str
             if scrape_url[-1:] == '/' and append_str[0:1] == '/':
                 append_str = append_str[1:]
@@ -289,7 +313,7 @@ class DjangoSpider(DjangoBaseSpider):
                 self.start_urls.insert(0, scrape_url)
                 self.pages.insert(0, "")
         
-        if self.scraper.pagination_type == 'N':
+        if self.scraper.pagination_type in ['N', 'O',]:
             self.start_urls.append(scrape_url)
             self.pages = ["",]
         num = len(self.start_urls)
@@ -299,36 +323,53 @@ class DjangoSpider(DjangoBaseSpider):
             url_str = 'URLs'
         self.log("Scraper set to run on {num} start {url_str}.".format(
             num=num, url_str=url_str), logging.INFO)
-
-
+    
+    
+    def _prepare_mp_req_data(self, kwargs_orig, form_data_orig, page, follow_page=''):
+        kwargs = kwargs_orig.copy()
+        if 'meta' not in kwargs:
+                kwargs['meta'] = {}
+        form_data = None
+        if form_data_orig:
+            form_data = json.loads(form_data_orig).copy()
+        if 'headers' in kwargs:
+            kwargs['headers'] = json.loads(json.dumps(kwargs['headers']).replace('{page}', str(page)))
+            kwargs['headers'] = json.loads(json.dumps(kwargs['headers']).replace('{follow_page}', str(follow_page)))
+        if 'body' in kwargs:
+            kwargs['body'] = kwargs['body'].replace('{page}', str(page))
+            kwargs['body'] = kwargs['body'].replace('{follow_page}', str(follow_page))
+        if 'cookies' in kwargs:
+            kwargs['cookies'] = json.loads(json.dumps(kwargs['cookies']).replace('{page}', str(page)))
+            kwargs['cookies'] = json.loads(json.dumps(kwargs['cookies']).replace('{follow_page}', str(follow_page)))
+        if form_data:
+            form_data = json.loads(json.dumps(form_data).replace('{page}', str(page)))
+            form_data = json.loads(json.dumps(form_data).replace('{follow_page}', str(follow_page)))
+        return kwargs, form_data
+    
+    
+    def _log_page_info(self, page_num, follow_page_num, url, rpt, form_data, kwargs):
+        self.dds_logger.info('')
+        self.dds_logger.info(self.bcolors['BOLD'] + '======================================================================================' + self.bcolors['ENDC'])
+        self.struct_log("{es}{es2}Scraping data from page {p}({fp}).{ec}{ec}".format(
+            p=page_num, fp=follow_page_num, es=self.bcolors['BOLD'], es2=self.bcolors['HEADER'], ec=self.bcolors['ENDC']))
+        self.struct_log("URL     : {url}".format(url=url))
+        self._log_request_info(rpt, form_data, kwargs)
+        self.dds_logger.info(self.bcolors['BOLD'] + '======================================================================================' + self.bcolors['ENDC'])
+    
+    
     def start_requests(self):
         index = 0
+        rpt = self.scraper.get_main_page_rpt()
+        follow_page_num = 0
+        
         for url in self.start_urls:
             self._set_meta_splash_args()
-            kwargs = self.mp_request_kwargs.copy()
-            if self.mp_form_data:
-                form_data = self.mp_form_data.copy()
-            else:
-                form_data = None
-            if 'headers' in kwargs:
-                kwargs['headers'] = json.loads(json.dumps(kwargs['headers']).replace('{page}', str(self.pages[index])))
-            if 'body' in kwargs:
-                kwargs['body'] = kwargs['body'].replace('{page}', str(self.pages[index]))
-            if 'cookies' in kwargs:
-                kwargs['cookies'] = json.loads(json.dumps(kwargs['cookies']).replace('{page}', str(self.pages[index])))
-            if form_data:
-                form_data = json.loads(json.dumps(form_data).replace('{page}', str(self.pages[index])))
-            if 'meta' not in kwargs:
-                    kwargs['meta'] = {}
-            kwargs['meta']['page'] = index + 1
-            rpt = self.scraper.get_main_page_rpt()
-            self.dds_logger.info('')
-            self.dds_logger.info(self.bcolors['BOLD'] + '======================================================================================' + self.bcolors['ENDC'])
-            self.struct_log("{es}{es2}Scraping data from page {page}.{ec}{ec}".format(
-                page=index+1, es=self.bcolors['BOLD'], es2=self.bcolors['HEADER'], ec=self.bcolors['ENDC']))
-            self.struct_log("URL     : {url}".format(url=url))
-            self._log_request_info(rpt, kwargs)
-            self.dds_logger.info(self.bcolors['BOLD'] + '======================================================================================' + self.bcolors['ENDC'])
+            page_num = index + 1
+            kwargs, form_data = self._prepare_mp_req_data(self.mp_request_kwargs, self.scraper.get_main_page_rpt().form_data, self.pages[index])
+            kwargs['meta']['page_num'] = page_num
+            kwargs['meta']['follow_page_num'] = follow_page_num
+            kwargs['meta']['rpt'] = rpt
+            self._log_page_info(page_num, follow_page_num, url, rpt, form_data, kwargs)
             index += 1
             if rpt.request_type == 'R':
                 yield Request(url, callback=self.parse, method=rpt.method, dont_filter=rpt.dont_filter, **kwargs)
@@ -403,7 +444,8 @@ class DjangoSpider(DjangoBaseSpider):
     
 
     def _scrape_item_attr(self, scraper_elem, response, from_page, item_num):
-        if(from_page == scraper_elem.request_page_type):
+        if(from_page == scraper_elem.request_page_type or 
+            (from_page == 'FP' and scraper_elem.request_page_type == 'MP')):
             procs = self._get_processors(scraper_elem)
             self._set_loader_context(scraper_elem.proc_ctxt)
             
@@ -435,14 +477,15 @@ class DjangoSpider(DjangoBaseSpider):
             if rpt.render_javascript:
                 rpt_str += '-JS'
             rpt_str += '|' + rpt.method
-            page_str = str(response.request.meta['page']) + '-'
+            page_str = str(response.request.meta['page_num'])
+            page_str += '(' + str(response.request.meta['follow_page_num']) + ')-'
             msg  = '{page_type: <4} {rpt_str: <13} {cs}{name: <20}{ce} {page}{num} '.format(page=page_str, num=str(item_num), name=name, rpt_str=rpt_str, page_type=from_page, cs=self.bcolors["BOLD"], ce=self.bcolors["ENDC"])
             c_values = loader.get_collected_values(name)
             if len(c_values) > 0:
                 val_str = c_values[0]
                 if self.conf['CONSOLE_LOG_LEVEL'] != 'DEBUG':
                     val_str = (val_str[:400] + '..') if len(val_str) > 400 else val_str
-                msg += smart_text(val_str) + "'"
+                msg += smart_text(val_str)
             else:
                 msg += 'None'
             self.log(msg, logging.INFO)
@@ -451,7 +494,7 @@ class DjangoSpider(DjangoBaseSpider):
     def _set_loader(self, response, from_page, xs, item):
         self.from_page = from_page
         rpt = self.scraper.get_rpt(from_page)
-        if not self.from_page == 'MP':
+        if not (self.from_page == 'MP' or self.from_page == 'FP'):
             item = response.request.meta['item']
             if rpt.content_type == 'J':
                 json_resp = json.loads(response.body_as_unicode())
@@ -470,7 +513,7 @@ class DjangoSpider(DjangoBaseSpider):
     def _set_dummy_loader(self, response, from_page, xs, item):
         self.from_page = from_page
         rpt = self.scraper.get_rpt(from_page)
-        if not self.from_page == 'MP':
+        if not (self.from_page == 'MP' or self.from_page == 'FP'):
             item = response.request.meta['item']
             if rpt.content_type == 'J':
                 json_resp = json.loads(response.body_as_unicode())
@@ -494,7 +537,7 @@ class DjangoSpider(DjangoBaseSpider):
             item_num = response.request.meta['item_num']
         self._set_loader(response, from_page, xs, self.scraped_obj_item_class())
         self._set_dummy_loader(response, from_page, xs, self.scraped_obj_item_class())
-        if from_page == 'MP':
+        if from_page == 'MP' or from_page == 'FP':
             self.items_read_count += 1
         else:
             if self.current_output_num_dp_response_bodies < self.conf['OUTPUT_NUM_DP_RESPONSE_BODIES']:
@@ -518,7 +561,7 @@ class DjangoSpider(DjangoBaseSpider):
                self.scraped_obj_class()._meta.get_field(key).blank and \
                not self.scraped_obj_class()._meta.get_field(key).null:
                 item[key] = ''
-        if from_page != 'MP':
+        if not (from_page == 'MP' or from_page == 'FP'):
             item, is_double = self._check_for_double_item(item)
             if response.request.meta['last']:
                 self.non_db_results[id(item)] = self.tmp_non_db_results[item_num].copy()
@@ -526,13 +569,13 @@ class DjangoSpider(DjangoBaseSpider):
         else:
             return item
     
-    def _replace_placeholders(self, text_str, item, item_num):
+    def _replace_placeholders(self, text_str, item, item_num, only_mp):
         applied = []
         if type(text_str).__name__ != 'str' and type(text_str).__name__ != 'unicode':
             return text_str, applied
         standard_elems = self.scraper.get_standard_elems()
         for scraper_elem in standard_elems:
-            if scraper_elem.request_page_type == 'MP':
+            if not only_mp or scraper_elem.request_page_type == 'MP':
                 name = scraper_elem.scraped_obj_attr.name
                 placeholder = '{' + name + '}'
                 if not scraper_elem.scraped_obj_attr.save_to_db:
@@ -550,28 +593,56 @@ class DjangoSpider(DjangoBaseSpider):
         return text_str, applied
 
 
+    def _do_req_info_replacements(self, item, item_num, page, json_dict, info_str):
+        json_dict = json.loads(json.dumps(json_dict).replace('{page}', str(page)))
+        for key, value in list(json_dict.items()):
+            new_value, applied = self._replace_placeholders(value, item, item_num, True)
+            json_dict[key] = new_value
+            if len(applied) > 0:
+                msg = "Request info placeholder(s) applied (item {id}): {a}".format(
+                    a=str(applied), id=item._dds_id_str)
+                self.log(msg, logging.DEBUG)
+                self.log(info_str + " [" + str(key) + "] before: " + str(value), logging.DEBUG)
+                self.log(info_str + " [" + str(key) + "] after : " + str(new_value), logging.DEBUG)
+        return json_dict
+    
+    
+
     def parse(self, response):
         xs = Selector(response)
+        base_objects = []
         base_elem = self.scraper.get_base_elem()
+        rpt = response.request.meta['rpt']
         
-        if self.current_output_num_mp_response_bodies < self.conf['OUTPUT_NUM_MP_RESPONSE_BODIES']:
-            self.current_output_num_mp_response_bodies += 1
-            self.log("Response body ({url})\n\n***** RP_MP_{num}_START *****\n{resp_body}\n***** RP_MP_{num}_END *****\n\n".format(
-                url=response.url,
-                resp_body=response.body,
-                num=self.current_output_num_mp_response_bodies), logging.INFO)
+        page_num = response.request.meta['page_num']
+        page = self.pages[page_num - 1]
+        follow_page_num = response.request.meta['follow_page_num']
         
-        if self.scraper.get_main_page_rpt().content_type == 'J':
-            json_resp = json.loads(response.body_as_unicode())
+        if rpt.page_type == 'MP':
+            if self.current_output_num_mp_response_bodies < self.conf['OUTPUT_NUM_MP_RESPONSE_BODIES']:
+                self.current_output_num_mp_response_bodies += 1
+                self.log("Response body ({url})\n\n***** RP_MP_{num}_START *****\n{resp_body}\n***** RP_MP_{num}_END *****\n\n".format(
+                    url=response.url,
+                    resp_body=response.body,
+                    num=self.current_output_num_mp_response_bodies), logging.INFO)
+        
+        if rpt.content_type == 'J':
+            json_resp = None
             try:
-                jsonpath_expr = parse(base_elem.x_path)
-            except JsonPathLexerError:
-                msg = "JsonPath for base elem could not be processed!"
-                self.dds_logger.error(msg)
-                raise CloseSpider()
-            base_objects = [match.value for match in jsonpath_expr.find(json_resp)]
-            if len(base_objects) > 0:
-                base_objects = base_objects[0]
+                json_resp = json.loads(response.body_as_unicode())
+            except ValueError:
+                msg = "JSON response for MP could not be parsed!"
+                self.log(msg, logging.ERROR)
+            if json_resp:
+                try:
+                    jsonpath_expr = parse(base_elem.x_path)
+                except JsonPathLexerError:
+                    msg = "JsonPath for base elem could not be processed!"
+                    self.dds_logger.error(msg)
+                    raise CloseSpider()
+                base_objects = [match.value for match in jsonpath_expr.find(json_resp)]
+                if len(base_objects) > 0:
+                    base_objects = base_objects[0]
         else:
             base_objects = response.xpath(base_elem.x_path)
 
@@ -587,14 +658,18 @@ class DjangoSpider(DjangoBaseSpider):
         for obj in base_objects:
             item_num = self.items_read_count + 1
             self.tmp_non_db_results[item_num] = {}
+            page_str = str(page_num) + '(' + str(follow_page_num) + ')'
             self.dds_logger.info("")
             self.dds_logger.info(self.bcolors['BOLD'] + '--------------------------------------------------------------------------------------' + self.bcolors['ENDC'])
             self.struct_log("{cs}Starting to crawl item {i} from page {p}.{ce}".format(
-                i=str(item_num), p=str(response.request.meta['page']), cs=self.bcolors["HEADER"], ce=self.bcolors["ENDC"]))
+                i=str(item_num), p=page_str, cs=self.bcolors["HEADER"], ce=self.bcolors["ENDC"]))
             self.dds_logger.info(self.bcolors['BOLD'] + '--------------------------------------------------------------------------------------' + self.bcolors['ENDC'])
-            item = self.parse_item(response, obj, 'MP', item_num)
-            item._dds_item_page = response.request.meta['page']
+            item = self.parse_item(response, obj, rpt.page_type, item_num)
+            item._dds_item_page = page
+            item._dds_item_page_num = page_num
+            item._dds_item_follow_page_num = follow_page_num
             item._dds_item_id = item_num
+            item._dds_id_str = str(item._dds_item_page_num) + '(' + str(item._dds_item_follow_page_num) + ')-' + str(item._dds_item_id)
             
             if item:
                 only_main_page_idfs = True
@@ -624,67 +699,49 @@ class DjangoSpider(DjangoBaseSpider):
                     for url_elem in url_elems:
                         if not url_elem.scraped_obj_attr.save_to_db:
                             url_before = self.tmp_non_db_results[item_num][url_elem.scraped_obj_attr.name]
-                            url, applied = self._replace_placeholders(url_before, item, item_num)
+                            url, applied = self._replace_placeholders(url_before, item, item_num, True)
                             self.tmp_non_db_results[item_num][url_elem.scraped_obj_attr.name] = url
                         else:
                             url_before = item[url_elem.scraped_obj_attr.name]
-                            url, applied = self._replace_placeholders(url_before, item, item_num)
+                            url, applied = self._replace_placeholders(url_before, item, item_num, True)
                             item[url_elem.scraped_obj_attr.name] = url
                         if len(applied) > 0:
-                            msg = "Detail page URL placeholder(s) applied (item {p}-{n}): {a}".format(
-                                a=str(applied), p=item._dds_item_page, n=item._dds_item_id)
+                            msg = "Detail page URL placeholder(s) applied (item {id}): {a}".format(
+                                a=str(applied), id=item._dds_id_str)
                             self.log(msg, logging.DEBUG)
                             self.log("URL before: " + url_before, logging.DEBUG)
                             self.log("URL after : " + url, logging.DEBUG)
-                        rpt = self.scraper.get_rpt_for_scraped_obj_attr(url_elem.scraped_obj_attr)
-                        kwargs = self.dp_request_kwargs[rpt.page_type].copy()
+                        dp_rpt = self.scraper.get_rpt_for_scraped_obj_attr(url_elem.scraped_obj_attr)
+                        kwargs = self.dp_request_kwargs[dp_rpt.page_type].copy()
                         
                         if 'meta' not in kwargs:
                             kwargs['meta'] = {}
+                        kwargs['meta']['page_num'] = page_num
+                        kwargs['meta']['follow_page_num'] = follow_page_num
                         kwargs['meta']['item'] = item
-                        kwargs['meta']['from_page'] = rpt.page_type
+                        kwargs['meta']['from_page'] = dp_rpt.page_type
                         kwargs['meta']['item_num'] = item_num
-                        kwargs['meta']['page'] = response.request.meta['page']
+                        
+                        kwargs['meta']['rpt'] = dp_rpt
                         
                         if 'headers' in kwargs:
-                            for key, value in list(kwargs['headers'].items()):
-                                new_value, applied = self._replace_placeholders(value, item, item_num)
-                                kwargs['headers'][key] = new_value
-                                if len(applied) > 0:
-                                    msg = "Request info placeholder(s) applied (item {p}-{n}): {a}".format(
-                                        a=str(applied), p=item._dds_item_page, n=item._dds_item_id)
-                                    self.log(msg, logging.DEBUG)
-                                    self.log("HEADERS [" + str(key) + "] before: " + str(value), logging.DEBUG)
-                                    self.log("HEADERS [" + str(key) + "] after : " + str(new_value), logging.DEBUG)
+                            kwargs['headers'] = self._do_req_info_replacements(item, item_num, page, kwargs['headers'], "HEADERS")
                         if 'body' in kwargs:
                             body_before = kwargs['body']
-                            kwargs['body'], applied = self._replace_placeholders(kwargs['body'], item, item_num)
+                            kwargs['body'] = kwargs['body'].replace('{page}', str(page))
+                            kwargs['body'], applied = self._replace_placeholders(kwargs['body'], item, item_num, True)
                             if len(applied) > 0:
-                                msg = "Request info placeholder(s) applied (item {p}-{n}): {a}".format(
-                                    a=str(applied), p=item._dds_item_page, n=item._dds_item_id)
+                                msg = "Request info placeholder(s) applied (item {id}): {a}".format(
+                                    a=str(applied), id=item._dds_id_str)
                                 self.log(msg, logging.DEBUG)
                                 self.log("BODY before: " + body_before, logging.DEBUG)
                                 self.log("BODY after : " + kwargs['body'], logging.DEBUG)
                         if 'cookies' in kwargs:
-                            for key, value in list(kwargs['cookies'].items()):
-                                new_value, applied = self._replace_placeholders(value, item, item_num)
-                                kwargs['cookies'][key] = new_value
-                                if len(applied) > 0:
-                                    msg = "Request info placeholder(s) applied (item {p}-{n}): {a}".format(
-                                        a=str(applied), p=item._dds_item_page, n=item._dds_item_id)
-                                    self.log(msg, logging.DEBUG)
-                                    self.log("COOKIE [" + str(key) + "] before: " + str(value), logging.DEBUG)
-                                    self.log("COOKIE [" + str(key) + "] after : " + str(new_value), logging.DEBUG)
-                        if rpt.request_type == 'F' and rpt.page_type in self.dp_form_data:
-                            for key, value in list(self.dp_form_data[rpt.page_type].items()):
-                                new_value, applied = self._replace_placeholders(value, item, item_num)
-                                self.dp_form_data[rpt.page_type][key] = new_value
-                                if len(applied) > 0:
-                                    msg = "Request info placeholder(s) applied (item {p}-{n}): {a}".format(
-                                        a=str(applied), p=item._dds_item_page, n=item._dds_item_id)
-                                    self.log(msg, logging.DEBUG)
-                                    self.log("FORM DATA [" + str(key) + "] before: " + str(value), logging.DEBUG)
-                                    self.log("FORM DATA [" + str(key) + "] after : " + str(new_value), logging.DEBUG)
+                            kwargs['cookies'] = self._do_req_info_replacements(item, item_num, page, kwargs['cookies'], "COOKIES")
+                        form_data = None
+                        if dp_rpt.request_type == 'F' and dp_rpt.form_data:
+                            form_data = json.loads(dp_rpt.form_data).copy()
+                            form_data = self._do_req_info_replacements(item, item_num, page, form_data, "FORM DATA")
                         
                         if url_elem == url_elems[len(url_elems)-1]:
                             kwargs['meta']['last'] = True
@@ -693,24 +750,71 @@ class DjangoSpider(DjangoBaseSpider):
                         self._set_meta_splash_args()
                         #logging.info(str(kwargs))
                         self.log(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", logging.INFO)
-                        msg = "{cs}Calling {dp} URL for item {p}-{n}...{ce}".format(
-                            dp=rpt.page_type, p=str(response.request.meta['page']), n=str(item_num),
+                        msg = "{cs}Calling {dp} URL for item {id}...{ce}".format(
+                            dp=dp_rpt.page_type, id=item._dds_id_str,
                             cs=self.bcolors["HEADER"], ce=self.bcolors["ENDC"])
                         self.log(msg, logging.INFO)
                         msg = "URL     : {url}".format(url=url)
                         self.log(msg, logging.INFO)
-                        self._log_request_info(rpt, kwargs)
+                        self._log_request_info(dp_rpt, form_data, kwargs)
                         self.log(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", logging.INFO)
                         
-                        if rpt.request_type == 'R':
-                            yield Request(url, callback=self.parse_item, method=rpt.method, dont_filter=rpt.dont_filter, **kwargs)
+                        if dp_rpt.request_type == 'R':
+                            yield response.follow(url, callback=self.parse_item, method=dp_rpt.method, dont_filter=dp_rpt.dont_filter, **kwargs)
                         else:
-                            yield FormRequest(url, callback=self.parse_item, method=rpt.method, formdata=self.dp_form_data[rpt.page_type], dont_filter=rpt.dont_filter, **kwargs)
+                            yield FormRequest(url, callback=self.parse_item, method=dp_rpt.method, formdata=form_data, dont_filter=dp_rpt.dont_filter, **kwargs)
+                for key, value in list(item.items()):
+                    #Fixing some extremely weird Python 2 encoding failure, 2017-06-29
+                    if type(value).__name__ == 'str':
+                        try:
+                            value = value.decode('utf-8')
+                        except AttributeError:
+                            pass
+                    if value and (type(value).__name__ in ['str', 'unicode']) and '{page}' in value:
+                        msg = "Applying page placeholder on {k}...".format(k=key)
+                        self.log(msg, logging.DEBUG)
+                        self.log("Value before: " + value, logging.DEBUG)
+                        value = value.replace('{page}', str(page))
+                        item[key] = value
+                        self.log("Value after: " + value, logging.DEBUG)
             else:
                 self.log("Item could not be read!", logging.ERROR)
+                
+        mir_reached = False
+        if self.conf['MAX_ITEMS_READ'] and (self.conf['MAX_ITEMS_READ'] - self.items_read_count <= 0):
+            mir_reached = True
+        if self.scraper.follow_pages_url_xpath and not mir_reached:
+            if not self.conf['NUM_PAGES_FOLLOW'] or follow_page_num < self.conf['NUM_PAGES_FOLLOW']:
+                url = response.xpath(self.scraper.follow_pages_url_xpath).extract_first()
+                if url is not None:
+                    self._set_meta_splash_args()
+                    follow_page = ''
+                    if self.scraper.follow_pages_page_xpath:
+                        follow_page = response.xpath(self.scraper.follow_pages_page_xpath).extract_first()
+                    form_data_orig = None
+                    if self.scraper.get_follow_page_rpts().count() > 0:
+                        f_rpt = self.scraper.get_follow_page_rpts()[0]
+                        form_data_orig = self.scraper.get_follow_page_rpts()[0].form_data
+                    else:
+                        f_rpt = self.scraper.get_main_page_rpt()
+                        form_data_orig = self.scraper.get_main_page_rpt().form_data
+                    kwargs, form_data = self._prepare_mp_req_data(self.fp_request_kwargs, form_data_orig, page, follow_page)
+                    
+                    follow_page_num += 1
+                    kwargs['meta']['page_num'] = page_num
+                    kwargs['meta']['follow_page_num'] = follow_page_num
+                    kwargs['meta']['rpt'] = f_rpt
+                    
+                    self._log_page_info(page_num, follow_page_num, url, f_rpt, form_data, kwargs)
+                    
+                    if f_rpt.request_type == 'R':
+                        yield response.follow(url, callback=self.parse, method=f_rpt.method, dont_filter=f_rpt.dont_filter, **kwargs)
+                    else:
+                        url = response.urljoin(url)
+                        yield FormRequest(url, callback=self.parse, method=f_rpt.method, formdata=form_data, dont_filter=f_rpt.dont_filter, **kwargs)
     
     
-    def _log_request_info(self, rpt, kwargs):
+    def _log_request_info(self, rpt, form_data, kwargs):
         level = logging.DEBUG
         extra_info = False
         if 'headers' in kwargs:
@@ -722,12 +826,8 @@ class DjangoSpider(DjangoBaseSpider):
         if 'cookies' in kwargs:
             self.log("COOKIES   : " + str(kwargs['cookies']), level)
             extra_info = True
-        if rpt.request_type == 'F':
-            if rpt.page_type in self.dp_form_data:
-                self.log("FORM DATA : " + str(self.dp_form_data[rpt.page_type]), level)
-                extra_info = True
-            if rpt.page_type == 'MP':
-                self.log("FORM DATA : " + str(self.mp_form_data), level)
+        if rpt.request_type == 'F' and form_data:
+                self.log("FORM DATA : " + str(form_data), level)
                 extra_info = True
         
         if not extra_info:
