@@ -1,29 +1,34 @@
+#Stage 2 Update (Python 3)
+from __future__ import unicode_literals
+from django.utils.encoding import python_2_unicode_compatible
 from django.db import models
 from django.db.models.signals import pre_delete
 from django.dispatch import receiver
-from scrapy.contrib.djangoitem import DjangoItem
+from scrapy_djangoitem import DjangoItem
 from dynamic_scraper.models import Scraper, SchedulerRuntime
 
 
+@python_2_unicode_compatible
 class NewsWebsite(models.Model):
     name = models.CharField(max_length=200)
     url = models.URLField()
     scraper = models.ForeignKey(Scraper, blank=True, null=True, on_delete=models.SET_NULL)
     scraper_runtime = models.ForeignKey(SchedulerRuntime, blank=True, null=True, on_delete=models.SET_NULL)
     
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
+@python_2_unicode_compatible
 class Article(models.Model):
     title = models.CharField(max_length=200)
     news_website = models.ForeignKey(NewsWebsite) 
     description = models.TextField(blank=True)
-    url = models.URLField()
-    thumbnail = models.CharField(max_length=200)
+    url = models.URLField(blank=True)
+    thumbnail = models.CharField(max_length=200, blank=True)
     checker_runtime = models.ForeignKey(SchedulerRuntime, blank=True, null=True, on_delete=models.SET_NULL)
     
-    def __unicode__(self):
+    def __str__(self):
         return self.title
 
 
