@@ -19,9 +19,8 @@ class DjangoWriterPipeline(object):
                 
                 item.save()
                 spider.action_successful = True
-                dds_id_str = str(item._dds_item_page) + '-' + str(item._dds_item_id)
                 spider.struct_log("{cs}Item {id} saved to Django DB.{ce}".format(
-                    id=dds_id_str,
+                    id=item._dds_id_str,
                     cs=spider.bcolors['OK'],
                     ce=spider.bcolors['ENDC']))
                     
