@@ -1,11 +1,5 @@
-"""
-This file demonstrates writing tests using the unittest module. These will pass
-when you run "manage.py test".
-
-Replace this with more appropriate tests for your application.
-"""
-
-import datetime
+#Stage 2 Update (Python 3)
+import datetime, os, time
 from django.test import TestCase
 from dynamic_scraper.utils import processors
 
@@ -19,6 +13,17 @@ class ProcessorsTest(TestCase):
         self.assertEqual(result_str, 'This text makes no sense!')
     
     
+    def test_remove_chars(self):
+        result_str = processors.remove_chars("Example Text!!!", {'remove_chars': '[-\.]+'})
+        self.assertEqual(result_str, 'Example Text!!!')
+        
+        result_str = processors.remove_chars("Example... Text-!-!!", {'remove_chars': '[-\.]+'})
+        self.assertEqual(result_str, 'Example Text!!!')
+        
+        result_str = processors.remove_chars("", {'remove_chars': '[-\.]+'})
+        self.assertEqual(result_str, '')
+    
+    
     def test_pre_string(self):
         result_str = processors.pre_string("text", {'pre_string': 'before_'})
         self.assertEqual(result_str, 'before_text')
@@ -82,6 +87,20 @@ class ProcessorsTest(TestCase):
         result_str = processors.time('22 Uhr 15', {'time': '%H Uhr %M'})
         self.assertEqual(result_str, '22:15:00')
     
+
+    def test_ts_to_date(self):
+        os.environ['TZ'] = 'Europe/Berlin'
+        time.tzset()
+        result_str = processors.ts_to_date('1434560700', {})
+        self.assertEqual(result_str, '2015-06-17')
+
+
+    def test_ts_to_time(self):
+        os.environ['TZ'] = 'Europe/Berlin'
+        time.tzset()
+        result_str = processors.ts_to_time('1434560700', {})
+        self.assertEqual(result_str, '19:05:00')
+
     
     def test_duration(self):
         result_str = processors.duration('01:25', {'duration': '%H:%M'})
