@@ -10,4 +10,8 @@ for test in `echo $tests`
 do
     echo $app.$test
     python manage.py test $app.$test
+    if [ "$?" -gt 0 ]
+    then
+        exit 1
+    fi
 done
\ No newline at end of file
