@@ -21,7 +21,11 @@ do
   for test in `echo ${!var}`
   do
       echo $suite.$test
-      python manage.py test $suite.$test &
+      python manage.py test $suite.$test
+      if [ "$?" -gt 0 ]
+      then
+          exit 1
+      fi
       sleep 0.2
   done
   wait
