@@ -11,4 +11,8 @@ for test in `echo $tests`
 do
     echo $suite.$test
     python manage.py test $suite.$test
+    if [ "$?" -gt 0 ]
+    then
+        exit 1
+    fi
 done
\ No newline at end of file
